@@ -27,6 +27,7 @@
 #include <fcitx-config/fcitx-config.h>
 #include <fcitx/addon.h>
 #include "sub_config_parser.h"
+#include "dummy_config.h"
 
 G_BEGIN_DECLS
 
@@ -47,17 +48,30 @@ G_BEGIN_DECLS
 #define FCITX_CONFIG_WIDGET_GET_CLASS(obj) \
     (G_TYPE_INSTANCE_GET_CLASS ((obj), FCITX_TYPE_CONFIG_WIDGET, FcitxConfigWidgetClass))
 
+
+enum UIType {
+    CW_Simple = 0x1,
+    CW_Full = 0x2,
+    CW_NoShow = 0x0
+};
+
 typedef struct {
-    GtkBox parent;
+    GtkGrid parent;
     FcitxConfigFileDesc* cfdesc;
     gchar* prefix;
     gchar* name;
     FcitxSubConfigParser* parser;
-    FcitxGenericConfig gconfig;
+    GHashTable* argmap;
+    enum UIType fullUiType;
+    enum UIType simpleUiType;
+    GtkWidget* simpleWidget;
+    GtkWidget* fullWidget;
+    GtkWidget* advanceCheckBox;
+    DummyConfig* config;
 } FcitxConfigWidget;
 
 typedef struct {
-    GtkBoxClass parent_class;
+    GtkGridClass parent_class;
 } FcitxConfigWidgetClass;
 
 typedef enum {
@@ -68,7 +82,7 @@ typedef enum {
 
 GType fcitx_config_widget_get_type(void);
 
-FcitxConfigWidget* fcitx_config_widget_new(FcitxConfigFileDesc* cfdesc, const gchar* prefix, const gchar* name, const char* subconfig);
+FcitxConfigWidget* fcitx_config_widget_new(FcitxConfigFileDesc* cfdesc, const gchar* prefix, const gchar* name, const gchar* subconfig);
 
 void fcitx_config_widget_response(FcitxConfigWidget* config_widget, ConfigWidgetAction action);
 
