@@ -26,28 +26,25 @@
 #include <fcitx-config/hotkey.h>
 
 #define _(s) gettext(s)
-//定义枚举类型，说明信号的名称和次序
+
 enum {
     KEYGRAB_BUTTON_CHANGED,
-    KEYGRAB_BUTTON_CURRENT_CHANGED,
     LAST_SIGNAL
 };
 static gint keygrab_button_signals[LAST_SIGNAL] = { 0 };
-static void keygrab_button_init(KeyGrabButton *keygrab_button);
+static void keygrab_button_init(KeyGrabButton* self);
 static void keygrab_button_class_init(KeyGrabButtonClass *keygrabbuttonclass);
 static void begin_key_grab(KeyGrabButton* self, gpointer v);
 static void end_key_grab(KeyGrabButton *self);
 static GtkWidget* popup_new(GtkWidget* parent, const gchar* text, gboolean mouse);
-static void on_key_press_event(GtkWidget *self, GdkEventKey *event, gpointer v);
+static void on_key_release_event(GtkWidget *self, GdkEventKey *event, gpointer v);
 
 G_DEFINE_TYPE(KeyGrabButton, keygrab_button, GTK_TYPE_BUTTON)
 
-static void keygrab_button_init(KeyGrabButton *keygrabbutton)
+static void keygrab_button_init(KeyGrabButton *self)
 {
-    keygrab_button_set_key(keygrabbutton, 0, 0);
-    gtk_widget_set_size_request(GTK_WIDGET(keygrabbutton), 150, -1);
-
-    g_signal_connect(G_OBJECT(keygrabbutton), "clicked", (GCallback) begin_key_grab, NULL);
+    self->allow_modifier_only = FALSE;
+    self->disallow_modifier_less = FALSE;
 }
 
 static void keygrab_button_class_init(KeyGrabButtonClass *keygrabbuttonclass)
@@ -57,14 +54,7 @@ static void keygrab_button_class_init(KeyGrabButtonClass *keygrabbuttonclass)
     keygrab_button_signals[KEYGRAB_BUTTON_CHANGED] = g_signal_new("changed",
             G_TYPE_FROM_CLASS(object_class),
             G_SIGNAL_RUN_FIRST,
-            G_STRUCT_OFFSET(KeyGrabButtonClass, changed),
-            NULL, NULL,
-            g_cclosure_marshal_VOID__VOID,
-            G_TYPE_NONE, 0, NULL);
-    keygrab_button_signals[KEYGRAB_BUTTON_CURRENT_CHANGED] = g_signal_new("current-changed",
-            G_TYPE_FROM_CLASS(object_class),
-            G_SIGNAL_RUN_FIRST,
-            G_STRUCT_OFFSET(KeyGrabButtonClass, current_changed),
+            0,
             NULL, NULL,
             g_cclosure_marshal_VOID__VOID,
             G_TYPE_NONE, 0, NULL);
@@ -72,7 +62,15 @@ static void keygrab_button_class_init(KeyGrabButtonClass *keygrabbuttonclass)
 //创建新的自定义控件
 GtkWidget* keygrab_button_new(void)
 {
-    return GTK_WIDGET(g_object_new(TYPE_KEYGRAB_BUTTON, 0));
+    KeyGrabButton *self = KEYGRAB_BUTTON(g_object_new(TYPE_KEYGRAB_BUTTON,
+                                                      "label", _("Empty"),
+                                                      NULL));
+    self->key = 0;
+    self->mods = 0;
+    gtk_widget_set_size_request(GTK_WIDGET(self), 150, -1);
+
+    g_signal_connect(G_OBJECT(self), "clicked", (GCallback) begin_key_grab, NULL);
+    return GTK_WIDGET(self);
 }
 
 
@@ -83,7 +81,7 @@ void begin_key_grab(KeyGrabButton* self, gpointer v)
     gtk_widget_add_events(GTK_WIDGET(b->popup), GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
     gtk_widget_show_all(b->popup);
     gtk_window_present(GTK_WINDOW(b->popup));
-    b->handler = g_signal_connect(G_OBJECT(b->popup), "key-press-event", (GCallback)on_key_press_event, b);
+    b->handler = g_signal_connect(G_OBJECT(b->popup), "key-release-event", (GCallback)on_key_release_event, b);
 
     GdkWindow* window = gtk_widget_get_window(GTK_WIDGET(b->popup));
     GdkDisplay* display = gdk_window_get_display (window);
@@ -114,7 +112,7 @@ void end_key_grab(KeyGrabButton *self)
     gtk_widget_destroy(b->popup);
 }
 
-void on_key_press_event(GtkWidget *self, GdkEventKey *event, gpointer v)
+void on_key_release_event(GtkWidget *self, GdkEventKey *event, gpointer v)
 {
     KeyGrabButton* b = KEYGRAB_BUTTON(v);
     guint key;
@@ -122,8 +120,6 @@ void on_key_press_event(GtkWidget *self, GdkEventKey *event, gpointer v)
 
     if ((event->keyval == GDK_KEY_Escape
             || event->keyval == GDK_KEY_Return) && !mods) {
-        if (event->keyval == GDK_KEY_Escape)
-            g_signal_emit_by_name(G_OBJECT(b), "changed", b->key, b->mods);
         end_key_grab(b);
         keygrab_button_set_key(b, 0, 0);
         return;
@@ -133,17 +129,34 @@ void on_key_press_event(GtkWidget *self, GdkEventKey *event, gpointer v)
     if (key == GDK_KEY_ISO_Left_Tab)
         key = GDK_KEY_Tab;
 
-    if (gtk_accelerator_valid(key, mods)
-            || (key == GDK_KEY_Tab && mods)) {
-        keygrab_button_set_key(b, key, mods);
-        end_key_grab(b);
-        b->key = key;
-        b->mods = mods;
-        g_signal_emit_by_name(G_OBJECT(b), "changed", b->key, b->mods);
+    if (b->disallow_modifier_less && mods == 0) {
+        return;
+    }
+
+    if (!b->allow_modifier_only &&
+        (key == GDK_KEY_Shift_L
+      || key == GDK_KEY_Shift_R
+      || key == GDK_KEY_Control_L
+      || key == GDK_KEY_Control_R
+      || key == GDK_KEY_Alt_L
+      || key == GDK_KEY_Alt_R
+      || key == GDK_KEY_Super_L
+      || key == GDK_KEY_Shift_R)) {
         return;
     }
 
     keygrab_button_set_key(b, key, mods);
+    end_key_grab(b);
+}
+
+void keygrab_button_set_allow_modifier_only(KeyGrabButton* self, gboolean allow)
+{
+    self->allow_modifier_only = allow;
+}
+
+void keygrab_button_set_disallow_modifier_less(KeyGrabButton* self, gboolean disallow)
+{
+    self->disallow_modifier_less = disallow;
 }
 
 void keygrab_button_set_key(KeyGrabButton* self, guint key, GdkModifierType mods)
@@ -152,18 +165,22 @@ void keygrab_button_set_key(KeyGrabButton* self, guint key, GdkModifierType mods
         mods &= ~GDK_SUPER_MASK;
         mods |= FcitxKeyState_Super;
     }
-    KeyGrabButton* b = KEYGRAB_BUTTON(self);
     gchar *label;
-    b->key = key;
-    b->mods = mods;
+    if (self->key != key || self->mods != mods) {
+        self->key = key;
+        self->mods = mods;
+        g_signal_emit(G_OBJECT(self), keygrab_button_signals[KEYGRAB_BUTTON_CHANGED], 0);
+    }
+    else {
+        return;
+    }
 
-    label = FcitxHotkeyGetKeyString(key, mods);
+    label = FcitxHotkeyGetReadableKeyString(key, mods);
 
     if (label == NULL || strlen(label) == 0) {
-        gtk_button_set_label(GTK_BUTTON(b), _("Empty"));
+        gtk_button_set_label(GTK_BUTTON(self), _("Empty"));
     } else {
-        gchar* lb = label;
-        gtk_button_set_label(GTK_BUTTON(b), lb);
+        gtk_button_set_label(GTK_BUTTON(self), label);
     }
 
     if (label)
