@@ -31,6 +31,7 @@
 #include "config_widget.h"
 #include "configdesc.h"
 #include "im_widget.h"
+#include "ui_widget.h"
 
 enum {
     LIST_ADDON,
@@ -47,10 +48,17 @@ static void _fcitx_main_window_add_addon_page(FcitxMainWindow* self);
 
 static void _fcitx_main_window_add_im_page(FcitxMainWindow* self);
 
+static void _fcitx_main_window_add_ui_page(FcitxMainWindow* self);
+
 void _fcitx_main_window_add_page(FcitxMainWindow* self, const char* name, GtkWidget* widget, const char* stock);
 
 static void _fcitx_main_window_addon_selection_changed(GtkTreeSelection *selection, gpointer data);
 
+static void _fcitx_main_window_addon_row_activated(GtkTreeView       *tree_view,
+                                                   GtkTreePath       *path,
+                                                   GtkTreeViewColumn *column,
+                                                   gpointer           user_data);
+
 static void _fcitx_main_window_configure_button_clicked(GtkButton *button, gpointer data);
 
 static void _fcitx_main_window_enabled_data_func(GtkCellLayout   *cell_layout,
@@ -65,9 +73,6 @@ static void _fcitx_main_window_name_data_func(GtkCellLayout   *cell_layout,
         GtkTreeIter     *iter,
         gpointer         user_data);
 
-static void _fcitx_main_window_apply_button_clicked(GtkButton *button,
-        gpointer   user_data);
-
 static void _fcitx_main_window_toggled_cb(GtkCellRenderer *renderer,
         gchar* str_path,
         gpointer         user_data);
@@ -89,6 +94,7 @@ fcitx_main_window_init(FcitxMainWindow* self)
 
     _fcitx_main_window_add_im_page(self);
     _fcitx_main_window_add_config_file_page(self);
+    _fcitx_main_window_add_ui_page(self);
     _fcitx_main_window_add_addon_page(self);
 
     gtk_widget_set_size_request(GTK_WIDGET(self), -1, 400);
@@ -96,8 +102,8 @@ fcitx_main_window_init(FcitxMainWindow* self)
     gtk_box_pack_start(GTK_BOX(vbox), self->configNotebook, TRUE, TRUE, 0);
 
     gtk_container_add(GTK_CONTAINER(self), vbox);
-    gtk_window_set_icon_name(GTK_WINDOW(self), "fcitx-configtool");
-    gtk_window_set_title(GTK_WINDOW(self), _("Fcitx Config"));
+    gtk_window_set_icon_name(GTK_WINDOW(self), "fcitx");
+    gtk_window_set_title(GTK_WINDOW(self), _("Input Method Configuration"));
 }
 
 GtkWidget*
@@ -153,6 +159,28 @@ void _fcitx_main_window_addon_selection_changed(GtkTreeSelection *selection, gpo
     }
 }
 
+void _fcitx_main_window_addon_row_activated(GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn* column, gpointer user_data)
+{
+    GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
+    GtkTreeIter iter;
+    FcitxAddon *addon = NULL;
+    if (gtk_tree_model_get_iter(model, &iter, path)) {
+        gtk_tree_model_get(model, &iter,
+                           LIST_ADDON, &addon,
+                           -1);
+
+        GtkWidget* dialog = fcitx_config_dialog_new(addon, GTK_WINDOW(user_data));
+        if (dialog)
+            gtk_widget_show_all(GTK_WIDGET(dialog));
+    }
+}
+
+static void
+_fcitx_main_window_config_widget_changed(FcitxConfigWidget* widget, gpointer user_data)
+{
+    fcitx_config_widget_response(widget, CONFIG_WIDGET_SAVE);
+}
+
 void _fcitx_main_window_add_config_file_page(FcitxMainWindow* self)
 {
     GtkWidget* vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
@@ -170,10 +198,7 @@ void _fcitx_main_window_add_config_file_page(FcitxMainWindow* self)
     gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, TRUE, 0);
     g_object_set(G_OBJECT(hbuttonbox), "margin", 5, NULL);
 
-    GtkWidget* applybutton = gtk_button_new_from_stock(GTK_STOCK_APPLY);
-    gtk_box_pack_start(GTK_BOX(hbuttonbox), applybutton, TRUE, TRUE, 0);
-    g_signal_connect(G_OBJECT(applybutton), "clicked", G_CALLBACK(_fcitx_main_window_apply_button_clicked), config_widget);
-
+    g_signal_connect(config_widget, "changed", (GCallback) _fcitx_main_window_config_widget_changed, NULL);
 
     _fcitx_main_window_add_page(self, _("Global Config"), vbox, GTK_STOCK_PREFERENCES);
 }
@@ -184,6 +209,20 @@ void _fcitx_main_window_add_im_page(FcitxMainWindow* self)
     _fcitx_main_window_add_page(self, _("Input Method"), imwidget, GTK_STOCK_EDIT);
 }
 
+gboolean _ui_connect(gpointer user_data)
+{
+    fcitx_ui_widget_connect(FCITX_UI_WIDGET(user_data));
+    return false;
+}
+
+void _fcitx_main_window_add_ui_page(FcitxMainWindow* self)
+{
+    GtkWidget* uiwidget = fcitx_ui_widget_new();
+    _fcitx_main_window_add_page(self, _("Appearance"), uiwidget, GTK_STOCK_ABOUT);
+
+    g_idle_add_full(G_PRIORITY_DEFAULT_IDLE, (GSourceFunc)_ui_connect, g_object_ref(uiwidget), (GDestroyNotify) g_object_unref);
+}
+
 gboolean _filter_addon_func(GtkTreeModel *model,
                             GtkTreeIter  *iter,
                             gpointer      data)
@@ -198,7 +237,7 @@ gboolean _filter_addon_func(GtkTreeModel *model,
 
     gboolean flag = TRUE;
     if (addon) {
-        if ((addon->category == AC_FRONTEND || addon->advance) && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->advancecheckbox))) {
+        if ((addon->category == AC_FRONTEND || !addon->bEnabled || addon->advance) && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->advancecheckbox))) {
             return false;
         }
         flag = flag && (strlen(filter_text) == 0
@@ -212,7 +251,7 @@ gboolean _filter_addon_func(GtkTreeModel *model,
 void _fcitx_main_window_checkbox_changed(GtkToggleButton* button, gpointer user_data)
 {
     FcitxMainWindow* self = user_data;
-    gtk_cell_renderer_toggle_set_activatable(GTK_CELL_RENDERER_TOGGLE(self->togglecell), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->advancecheckbox)));
+    gtk_tree_view_column_set_visible(self->checkboxcolumn, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self->advancecheckbox)));
     gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(self->filtermodel));
 }
 
@@ -255,7 +294,6 @@ void _fcitx_main_window_add_addon_page(FcitxMainWindow* self)
     gtk_entry_set_placeholder_text(GTK_ENTRY (self->filterentry), _("Search Addon"));
 #endif
     g_signal_connect(G_OBJECT(self->filterentry), "icon-press", G_CALLBACK (icon_press_cb), NULL);
-    gtk_box_pack_start(GTK_BOX(vbox), self->filterentry, FALSE, TRUE, 5);
 
     /* list view */
     self->addonstore = gtk_list_store_new(N_COLUMNS, G_TYPE_POINTER);
@@ -277,15 +315,14 @@ void _fcitx_main_window_add_addon_page(FcitxMainWindow* self)
 
     /* add column check box */
     self->togglecell = gtk_cell_renderer_toggle_new();
-    gtk_cell_renderer_toggle_set_activatable(GTK_CELL_RENDERER_TOGGLE(self->togglecell), FALSE);
     self->checkboxcolumn = gtk_tree_view_column_new_with_attributes("Enable", self->togglecell, NULL);
+    gtk_tree_view_column_set_visible(self->checkboxcolumn, FALSE);
     gtk_tree_view_append_column(GTK_TREE_VIEW(self->addonview), self->checkboxcolumn);
     gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(self->checkboxcolumn),
                                        self->togglecell,
                                        _fcitx_main_window_enabled_data_func,
                                        NULL,
                                        NULL);
-    gtk_cell_renderer_toggle_set_activatable(GTK_CELL_RENDERER_TOGGLE(self->togglecell), FALSE);
 
     /* add column text */
     GtkCellRenderer* renderer = gtk_cell_renderer_text_new();
@@ -310,6 +347,7 @@ void _fcitx_main_window_add_addon_page(FcitxMainWindow* self)
 
     gtk_box_pack_start(GTK_BOX(vbox), self->advancecheckbox, FALSE, TRUE, 0);
     g_object_set(G_OBJECT(self->advancecheckbox), "margin-left", 5, "margin-right", 5, NULL);
+    gtk_box_pack_start(GTK_BOX(vbox), self->filterentry, FALSE, TRUE, 5);
 
     /* configure button */
     GtkWidget* hbuttonbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
@@ -329,6 +367,10 @@ void _fcitx_main_window_add_addon_page(FcitxMainWindow* self)
     g_signal_connect(G_OBJECT(self->filterentry), "changed", G_CALLBACK(_fcitx_main_window_filtertext_changed), self);
     gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(self->filtermodel));
 
+    g_signal_connect(G_OBJECT(self->addonview),
+                     "row-activated",
+                     G_CALLBACK(_fcitx_main_window_addon_row_activated), self);
+
     _fcitx_main_window_add_page(self, _("Addon"), vbox, GTK_STOCK_ADD);
 }
 
@@ -405,13 +447,6 @@ _fcitx_main_window_name_data_func(GtkCellLayout   *cell_layout,
     g_free(string);
 }
 
-
-void _fcitx_main_window_apply_button_clicked(GtkButton* button, gpointer user_data)
-{
-    FcitxConfigWidget* config_widget = user_data;
-    fcitx_config_widget_response(config_widget, CONFIG_WIDGET_SAVE);
-}
-
 void _fcitx_main_window_configure_button_clicked(GtkButton* button, gpointer data)
 {
     FcitxMainWindow* self = data;
